@@ -1,5 +1,5 @@
 /* Linker file opening and searching.
-   Copyright (C) 1991-2019 Free Software Foundation, Inc.
+   Copyright (C) 1991-2025 Free Software Foundation, Inc.
 
    This file is part of the GNU Binutils.
 
@@ -21,6 +21,7 @@
 #include "sysdep.h"
 #include "bfd.h"
 #include "bfdlink.h"
+#include "ctf-api.h"
 #include "safe-ctype.h"
 #include "ld.h"
 #include "ldmisc.h"
@@ -33,12 +34,13 @@
 #include "ldemul.h"
 #include "libiberty.h"
 #include "filenames.h"
-#ifdef ENABLE_PLUGINS
+#include <fnmatch.h>
+#if BFD_SUPPORTS_PLUGINS
 #include "plugin-api.h"
 #include "plugin.h"
-#endif /* ENABLE_PLUGINS */
+#endif /* BFD_SUPPORTS_PLUGINS */
 
-bfd_boolean ldfile_assumed_script = FALSE;
+bool ldfile_assumed_script = false;
 const char *ldfile_output_machine_name = "";
 unsigned long ldfile_output_machine;
 enum bfd_architecture ldfile_output_architecture;
@@ -61,25 +63,235 @@ typedef struct search_arch
 } search_arch_type;
 
 static search_dirs_type **search_tail_ptr = &search_head;
+static search_dirs_type *script_search;
 static search_arch_type *search_arch_head;
 static search_arch_type **search_arch_tail_ptr = &search_arch_head;
 
+typedef struct input_remap
+{
+  const char *          pattern;  /* Pattern to match input files.  */
+  const char *          renamed;  /* Filename to use if the pattern matches.  */
+  struct input_remap *  next;     /* Link in a chain of these structures.  */
+} input_remap;
+
+static struct input_remap * input_remaps = NULL;
+
+void
+ldfile_add_remap (const char * pattern, const char * renamed)
+{
+  struct input_remap * new_entry;
+
+  new_entry = xmalloc (sizeof * new_entry);
+  new_entry->pattern = xstrdup (pattern);
+  new_entry->next = NULL;
+
+  /* Look for special filenames that mean that the input file should be ignored.  */
+  if (strcmp (renamed, "/dev/null") == 0
+      || strcmp (renamed, "NUL") == 0)
+    new_entry->renamed = NULL;
+  else
+    /* FIXME: Should we add sanity checking of the 'renamed' string ?  */
+    new_entry->renamed = xstrdup (renamed);
+
+  /* It would be easier to add this new node at the start of the chain,
+     but users expect that remapping will occur in the order in which
+     they occur on the command line, and in the remapping files.  */
+  if (input_remaps == NULL)
+    {
+      input_remaps = new_entry;
+    }
+  else
+    {
+      struct input_remap * i;
+
+      for (i = input_remaps; i->next != NULL; i = i->next)
+	;
+      i->next = new_entry;
+    }
+}
+
+static void
+ldfile_remap_input_free (void)
+{
+  while (input_remaps != NULL)
+    {
+      struct input_remap * i = input_remaps;
+
+      input_remaps = i->next;
+      free ((void *) i->pattern);
+      free ((void *) i->renamed);
+      free (i);
+    }
+}
+
+bool
+ldfile_add_remap_file (const char * file)
+{
+  FILE * f;
+
+  f = fopen (file, FOPEN_RT);
+  if (f == NULL)
+    return false;
+
+  size_t linelen = 256;
+  char * line = xmalloc (linelen);
+
+  do
+    {
+      char * p = line;
+      char * q;
+
+      /* Normally this would use getline(3), but we need to be portable.  */
+      while ((q = fgets (p, linelen - (p - line), f)) != NULL
+	     && strlen (q) == linelen - (p - line) - 1
+	     && line[linelen - 2] != '\n')
+	{
+	  line = xrealloc (line, 2 * linelen);
+	  p = line + linelen - 1;
+	  linelen += linelen;
+	}
+
+      if (q == NULL && p == line)
+	break;
+
+      p = strchr (line, '\n');
+      if (p)
+	*p = '\0';
+
+      /* Because the file format does not know any form of quoting we
+	 can search forward for the next '#' character and if found
+	 make it terminating the line.  */
+      p = strchr (line, '#');
+      if (p)
+	*p = '\0';
+
+      /* Remove leading whitespace.  NUL is no whitespace character.  */
+      p = line;
+      while (*p == ' ' || *p == '\f' || *p == '\r' || *p == '\t' || *p == '\v')
+	++p;
+
+      /* If the line is blank it is ignored.  */
+      if (*p == '\0')
+	continue;
+
+      char * pattern = p;
+
+      /* Advance past the pattern.  We accept whitespace or '=' as an
+	 end-of-pattern marker.  */
+      while (*p && *p != '=' && *p != ' ' && *p != '\t' && *p != '\f'
+	     && *p != '\r' && *p != '\v')
+	++p;
+
+      if (*p == '\0')
+	{
+	  fatal ("%P: malformed remap file entry: %s\n", line);
+	  continue;
+	}
+
+      * p++ = '\0';
+
+      /* Skip whitespace again.  */
+      while (*p == ' ' || *p == '\f' || *p == '\r' || *p == '\t' || *p == '\v')
+	++p;
+
+      if (*p == '\0')
+	{
+	  fatal ("%P: malformed remap file entry: %s\n", line);
+	  continue;
+	}
+
+      char * renamed = p;
+
+      /* Advance past the rename entry.  */
+      while (*p && *p != '=' && *p != ' ' && *p != '\t' && *p != '\f'
+	     && *p != '\r' && *p != '\v')
+	++p;
+      /* And terminate it.  */
+      *p = '\0';
+
+      ldfile_add_remap (pattern, renamed);
+    }
+  while (! feof (f));
+
+  free (line);
+  fclose (f);
+
+  return true;
+}
+
+const char *
+ldfile_possibly_remap_input (const char * filename)
+{
+  struct input_remap * i;
+
+  if (filename == NULL)
+    return NULL;
+
+  for (i = input_remaps; i != NULL; i = i->next)
+    {
+      if (fnmatch (i->pattern, filename, 0) == 0)
+	{
+	  if (verbose)
+	    {
+	      if (strpbrk ((i->pattern), "?*[") != NULL)
+		{
+		  if (i->renamed)
+		    info_msg (_("remap input file '%s' to '%s' based upon pattern '%s'\n"),
+			      filename, i->renamed, i->pattern);
+		  else
+		    info_msg (_("remove input file '%s' based upon pattern '%s'\n"),
+			      filename, i->pattern);
+		}
+	      else
+		{
+		  if (i->renamed)
+		    info_msg (_("remap input file '%s' to '%s'\n"),
+			      filename, i->renamed);
+		  else
+		    info_msg (_("remove input file '%s'\n"),
+			      filename);
+		}
+	    }
+
+	  return i->renamed;
+	}
+    }
+	 
+  return filename;
+}
+
+void
+ldfile_print_input_remaps (void)
+{
+  if (input_remaps == NULL)
+    return;
+
+  minfo (_("\nInput File Remapping\n\n"));
+
+  struct input_remap * i;
+
+  for (i = input_remaps; i != NULL; i = i->next)
+    minfo (_("  Pattern: %s\tMaps To: %s\n"), i->pattern,
+	   i->renamed ? i->renamed : _("<discard>"));
+}
+
+
 /* Test whether a pathname, after canonicalization, is the same or a
    sub-directory of the sysroot directory.  */
 
-static bfd_boolean
+static bool
 is_sysrooted_pathname (const char *name)
 {
   char *realname;
   int len;
-  bfd_boolean result;
+  bool result;
 
   if (ld_canon_sysroot == NULL)
-    return FALSE;
+    return false;
 
   realname = lrealpath (name);
   len = strlen (realname);
-  result = FALSE;
+  result = false;
   if (len > ld_canon_sysroot_len
       && IS_DIR_SEPARATOR (realname[ld_canon_sysroot_len]))
     {
@@ -95,7 +307,7 @@ is_sysrooted_pathname (const char *name)
    Makes a copy of NAME using xmalloc().  */
 
 void
-ldfile_add_library_path (const char *name, bfd_boolean cmdline)
+ldfile_add_library_path (const char *name, bool cmdline)
 {
   search_dirs_type *new_dirs;
 
@@ -112,15 +324,27 @@ ldfile_add_library_path (const char *name, bfd_boolean cmdline)
      now.  */
   if (name[0] == '=')
     new_dirs->name = concat (ld_sysroot, name + 1, (const char *) NULL);
-  else if (CONST_STRNEQ (name, "$SYSROOT"))
+  else if (startswith (name, "$SYSROOT"))
     new_dirs->name = concat (ld_sysroot, name + strlen ("$SYSROOT"), (const char *) NULL);
   else
     new_dirs->name = xstrdup (name);
 }
 
+static void
+ldfile_library_path_free (search_dirs_type **root)
+{
+  search_dirs_type *ent;
+  while ((ent = *root) != NULL)
+    {
+      *root = ent->next;
+      free ((void *) ent->name);
+      free (ent);
+    }
+}
+
 /* Try to open a BFD for a lang_input_statement.  */
 
-bfd_boolean
+bool
 ldfile_try_open_bfd (const char *attempt,
 		     lang_input_statement_type *entry)
 {
@@ -137,17 +361,23 @@ ldfile_try_open_bfd (const char *attempt,
   if (entry->the_bfd == NULL)
     {
       if (bfd_get_error () == bfd_error_invalid_target)
-	einfo (_("%F%P: invalid BFD target `%s'\n"), entry->target);
-      return FALSE;
+	fatal (_("%P: invalid BFD target `%s'\n"), entry->target);
+      return false;
     }
 
+  /* PR 30568: Do not track lto generated temporary object files.  */
+#if BFD_SUPPORTS_PLUGINS
+  if (!entry->flags.lto_output)
+#endif
+    track_dependency_files (attempt);
+
   /* Linker needs to decompress sections.  */
   entry->the_bfd->flags |= BFD_DECOMPRESS;
 
   /* This is a linker input BFD.  */
   entry->the_bfd->is_linker_input = 1;
 
-#ifdef ENABLE_PLUGINS
+#if BFD_SUPPORTS_PLUGINS
   if (entry->flags.lto_output)
     entry->the_bfd->lto_output = 1;
 #endif
@@ -159,8 +389,8 @@ ldfile_try_open_bfd (const char *attempt,
      a dynamic object.
 
      In the code below, it's OK to exit early if the check fails,
-     closing the checked BFD and returning FALSE, but if the BFD
-     checks out compatible, do not exit early returning TRUE, or
+     closing the checked BFD and returning false, but if the BFD
+     checks out compatible, do not exit early returning true, or
      the plugins will not get a chance to claim the file.  */
 
   if (entry->flags.search_dirs || !entry->flags.dynamic)
@@ -188,9 +418,9 @@ ldfile_try_open_bfd (const char *attempt,
 		  /* Try to interpret the file as a linker script.  */
 		  ldfile_open_command_file (attempt);
 
-		  ldfile_assumed_script = TRUE;
+		  ldfile_assumed_script = true;
 		  parser_input = input_selected;
-		  ldlex_both ();
+		  ldlex_script ();
 		  token = INPUT_SCRIPT;
 		  while (token != 0)
 		    {
@@ -208,18 +438,11 @@ ldfile_try_open_bfd (const char *attempt,
 			  if (token == ',')
 			    {
 			      if ((token = yylex ()) != NAME)
-				{
-				  free (arg1);
-				  continue;
-				}
+				continue;
 			      arg2 = yylval.name;
 			      if ((token = yylex ()) != ','
 				  || (token = yylex ()) != NAME)
-				{
-				  free (arg1);
-				  free (arg2);
-				  continue;
-				}
+				continue;
 			      arg3 = yylval.name;
 			      token = yylex ();
 			    }
@@ -238,25 +461,18 @@ ldfile_try_open_bfd (const char *attempt,
 			      if (strcmp (arg, lang_get_output_target ()) != 0)
 				skip = 1;
 			    }
-			  free (arg1);
-			  if (arg2) free (arg2);
-			  if (arg3) free (arg3);
 			  break;
 			case NAME:
 			case LNAME:
 			case VERS_IDENTIFIER:
 			case VERS_TAG:
-			  free (yylval.name);
-			  break;
 			case INT:
-			  if (yylval.bigint.str)
-			    free (yylval.bigint.str);
 			  break;
 			}
 		      token = yylex ();
 		    }
 		  ldlex_popstate ();
-		  ldfile_assumed_script = FALSE;
+		  ldfile_assumed_script = false;
 		  fclose (yyin);
 		  yyin = NULL;
 		  if (skip)
@@ -267,7 +483,7 @@ ldfile_try_open_bfd (const char *attempt,
 			       attempt, entry->local_sym_name);
 		      bfd_close (entry->the_bfd);
 		      entry->the_bfd = NULL;
-		      return FALSE;
+		      return false;
 		    }
 		}
 	      goto success;
@@ -275,11 +491,11 @@ ldfile_try_open_bfd (const char *attempt,
 
 	  if (!entry->flags.dynamic && (entry->the_bfd->flags & DYNAMIC) != 0)
 	    {
-	      einfo (_("%F%P: attempted static link of dynamic object `%s'\n"),
+	      fatal (_("%P: attempted static link of dynamic object `%s'\n"),
 		     attempt);
 	      bfd_close (entry->the_bfd);
 	      entry->the_bfd = NULL;
-	      return FALSE;
+	      return false;
 	    }
 
 	  if (entry->flags.search_dirs
@@ -297,12 +513,12 @@ ldfile_try_open_bfd (const char *attempt,
 		       attempt, entry->local_sym_name);
 	      bfd_close (entry->the_bfd);
 	      entry->the_bfd = NULL;
-	      return FALSE;
+	      return false;
 	    }
 	}
     }
-success:
-#ifdef ENABLE_PLUGINS
+ success:
+#if BFD_SUPPORTS_PLUGINS
   /* If plugins are active, they get first chance to claim
      any successfully-opened input file.  We skip archives
      here; the plugin wants us to offer it the individual
@@ -316,17 +532,19 @@ success:
       && !no_more_claiming
       && bfd_check_format (entry->the_bfd, bfd_object))
     plugin_maybe_claim (entry);
-#endif /* ENABLE_PLUGINS */
+  else
+    cmdline_check_object_only_section (entry->the_bfd, false);
+#endif /* BFD_SUPPORTS_PLUGINS */
 
   /* It opened OK, the format checked out, and the plugins have had
      their chance to claim it, so this is success.  */
-  return TRUE;
+  return true;
 }
 
 /* Search for and open the file specified by ENTRY.  If it is an
    archive, use ARCH, LIB and SUFFIX to modify the file name.  */
 
-bfd_boolean
+bool
 ldfile_open_file_search (const char *arch,
 			 lang_input_statement_type *entry,
 			 const char *lib,
@@ -345,15 +563,15 @@ ldfile_open_file_search (const char *arch,
 	  if (ldfile_try_open_bfd (name, entry))
 	    {
 	      entry->filename = name;
-	      return TRUE;
+	      return true;
 	    }
 	  free (name);
 	}
       else if (ldfile_try_open_bfd (entry->filename, entry))
-	return TRUE;
+	return true;
 
       if (IS_ABSOLUTE_PATH (entry->filename))
-	return FALSE;
+	return false;
     }
 
   for (search = search_head; search != NULL; search = search->next)
@@ -363,7 +581,7 @@ ldfile_open_file_search (const char *arch,
       if (entry->flags.dynamic && !bfd_link_relocatable (&link_info))
 	{
 	  if (ldemul_open_dynamic_archive (arch, search, entry))
-	    return TRUE;
+	    return true;
 	}
 
       if (entry->flags.maybe_archive && !entry->flags.full_name_provided)
@@ -376,13 +594,13 @@ ldfile_open_file_search (const char *arch,
       if (ldfile_try_open_bfd (string, entry))
 	{
 	  entry->filename = string;
-	  return TRUE;
+	  return true;
 	}
 
       free (string);
     }
 
-  return FALSE;
+  return false;
 }
 
 /* Open the input file specified by ENTRY.
@@ -407,15 +625,32 @@ ldfile_open_file (lang_input_statement_type *entry)
       else
 	einfo (_("%P: cannot find %s: %E\n"), entry->local_sym_name);
 
-      entry->flags.missing_file = TRUE;
-      input_flags.missing_file = TRUE;
+      entry->flags.missing_file = true;
+      input_flags.missing_file = true;
     }
   else
     {
       search_arch_type *arch;
-      bfd_boolean found = FALSE;
+      bool found = false;
 
-      /* Try to open <filename><suffix> or lib<filename><suffix>.a */
+      /* If extra_search_path is set, entry->filename is a relative path.
+	 Search the directory of the current linker script before searching
+	 other paths. */
+      if (entry->extra_search_path)
+	{
+	  char *path = concat (entry->extra_search_path, slash, entry->filename,
+			       (const char *)0);
+	  if (ldfile_try_open_bfd (path, entry))
+	    {
+	      entry->filename = path;
+	      entry->flags.search_dirs = false;
+	      return;
+	    }
+
+	  free (path);
+	}
+
+      /* Try to open <filename><suffix> or lib<filename><suffix>.a.  */
       for (arch = search_arch_head; arch != NULL; arch = arch->next)
 	{
 	  found = ldfile_open_file_search (arch->name, entry, "lib", ".a");
@@ -434,7 +669,7 @@ ldfile_open_file (lang_input_statement_type *entry)
       /* If we have found the file, we don't need to search directories
 	 again.  */
       if (found)
-	entry->flags.search_dirs = FALSE;
+	entry->flags.search_dirs = false;
       else
 	{
 	  if (entry->flags.sysrooted
@@ -442,10 +677,66 @@ ldfile_open_file (lang_input_statement_type *entry)
 	       && IS_ABSOLUTE_PATH (entry->local_sym_name))
 	    einfo (_("%P: cannot find %s inside %s\n"),
 		   entry->local_sym_name, ld_sysroot);
+#if SUPPORT_ERROR_HANDLING_SCRIPT
+	  else if (error_handling_script != NULL)
+	    {
+	      char *        argv[4];
+	      const char *  res;
+	      int           status, err;
+
+	      argv[0] = error_handling_script;
+	      argv[1] = "missing-lib";
+	      argv[2] = (char *) entry->local_sym_name;
+	      argv[3] = NULL;
+      
+	      if (verbose)
+		einfo (_("%P: About to run error handling script '%s' with arguments: '%s' '%s'\n"),
+		       argv[0], argv[1], argv[2]);
+
+	      res = pex_one (PEX_SEARCH, error_handling_script, argv,
+			     N_("error handling script"),
+			     NULL /* Send stdout to random, temp file.  */,
+			     NULL /* Write to stderr.  */,
+			     &status, &err);
+	      if (res != NULL)
+		{
+		  einfo (_("%P: Failed to run error handling script '%s', reason: "),
+			 error_handling_script);
+		  /* FIXME: We assume here that errrno == err.  */
+		  perror (res);
+		}
+	      else /* We ignore the return status of the script
+		      and always print the error message.  */
+		einfo (_("%P: cannot find %s: %E\n"), entry->local_sym_name);
+	    }
+#endif
 	  else
-	    einfo (_("%P: cannot find %s\n"), entry->local_sym_name);
-	  entry->flags.missing_file = TRUE;
-	  input_flags.missing_file = TRUE;
+	    einfo (_("%P: cannot find %s: %E\n"), entry->local_sym_name);
+
+	  /* Be kind to users who are creating static executables, but
+	     have forgotten to install the necessary static libraries.  */
+	  if (entry->flags.dynamic == false && startswith (entry->local_sym_name, "-l"))
+	    einfo (_("%P: have you installed the static version of the %s library ?\n"),
+		     entry->local_sym_name + 2);
+
+	  /* PR 25747: Be kind to users who forgot to add the
+	     "lib" prefix to their library when it was created.  */
+	  for (arch = search_arch_head; arch != NULL; arch = arch->next)
+	    {
+	      if (ldfile_open_file_search (arch->name, entry, "", ".a"))
+		{
+		  const char * base = lbasename (entry->filename);
+
+		  einfo (_("%P: note to link with %s use -l:%s or rename it to lib%s\n"),
+			 entry->filename, base, base);
+		  bfd_close (entry->the_bfd);
+		  entry->the_bfd = NULL;
+		  break;
+		}
+	    }
+
+	  entry->flags.missing_file = true;
+	  input_flags.missing_file = true;
 	}
     }
 }
@@ -453,14 +744,17 @@ ldfile_open_file (lang_input_statement_type *entry)
 /* Try to open NAME.  */
 
 static FILE *
-try_open (const char *name, bfd_boolean *sysrooted)
+try_open (const char *name, bool *sysrooted)
 {
   FILE *result;
 
   result = fopen (name, "r");
 
   if (result != NULL)
-    *sysrooted = is_sysrooted_pathname (name);
+    {
+      *sysrooted = is_sysrooted_pathname (name);
+      track_dependency_files (name);
+    }
 
   if (verbose)
     {
@@ -475,12 +769,12 @@ try_open (const char *name, bfd_boolean *sysrooted)
 
 /* Return TRUE iff directory DIR contains an "ldscripts" subdirectory.  */
 
-static bfd_boolean
+static bool
 check_for_scripts_dir (char *dir)
 {
   char *buf;
   struct stat s;
-  bfd_boolean res;
+  bool res;
 
   buf = concat (dir, "/ldscripts", (const char *) NULL);
   res = stat (buf, &s) == 0 && S_ISDIR (s.st_mode);
@@ -535,13 +829,12 @@ find_scripts_dir (void)
 
 static FILE *
 ldfile_find_command_file (const char *name,
-			  bfd_boolean default_only,
-			  bfd_boolean *sysrooted)
+			  bool default_only,
+			  bool *sysrooted)
 {
   search_dirs_type *search;
   FILE *result = NULL;
   char *path;
-  static search_dirs_type *script_search;
 
   if (!default_only)
     {
@@ -558,8 +851,9 @@ ldfile_find_command_file (const char *name,
 	{
 	  search_dirs_type **save_tail_ptr = search_tail_ptr;
 	  search_tail_ptr = &script_search;
-	  ldfile_add_library_path (script_dir, TRUE);
+	  ldfile_add_library_path (script_dir, true);
 	  search_tail_ptr = save_tail_ptr;
+	  free (script_dir);
 	}
     }
 
@@ -585,20 +879,44 @@ ldfile_find_command_file (const char *name,
   return result;
 }
 
+struct script_name_list *processed_scripts = NULL;
 /* Open command file NAME.  */
 
 static void
-ldfile_open_command_file_1 (const char *name, bfd_boolean default_only)
+ldfile_open_command_file_1 (const char *name, enum script_open_style open_how)
 {
   FILE *ldlex_input_stack;
-  bfd_boolean sysrooted;
+  bool sysrooted;
+  struct script_name_list *script;
+  size_t len;
 
-  ldlex_input_stack = ldfile_find_command_file (name, default_only, &sysrooted);
+  /* PR 24576: Catch the case where the user has accidentally included
+     the same linker script twice.  */
+  for (script = processed_scripts; script != NULL; script = script->next)
+    {
+      if ((open_how != script_nonT || script->open_how != script_nonT)
+	  && strcmp (name, script->name) == 0)
+	{
+	  fatal (_("%P: error: linker script file '%s'"
+		   " appears multiple times\n"), name);
+	  return;
+	}
+    }
 
+  len = strlen (name);
+  script = xmalloc (sizeof (*script) + len);
+  script->next = processed_scripts;
+  script->open_how = open_how;
+  memcpy (script->name, name, len + 1);
+  processed_scripts = script;
+
+  ldlex_input_stack = ldfile_find_command_file (name,
+						open_how == script_defaultT,
+						&sysrooted);
   if (ldlex_input_stack == NULL)
     {
       bfd_set_error (bfd_error_system_call);
-      einfo (_("%F%P: cannot open linker script file %s: %E\n"), name);
+      fatal (_("%P: cannot open linker script file %s: %E\n"), name);
       return;
     }
 
@@ -609,13 +927,30 @@ ldfile_open_command_file_1 (const char *name, bfd_boolean default_only)
   saved_script_handle = ldlex_input_stack;
 }
 
+static void
+ldfile_script_free (struct script_name_list **root)
+{
+  struct script_name_list *ent;
+  while ((ent = *root) != NULL)
+    {
+      *root = ent->next;
+      free (ent);
+    }
+}
+
 /* Open command file NAME in the current directory, -L directories,
    the default script location, in that order.  */
 
 void
 ldfile_open_command_file (const char *name)
 {
-  ldfile_open_command_file_1 (name, FALSE);
+  ldfile_open_command_file_1 (name, script_nonT);
+}
+
+void
+ldfile_open_script_file (const char *name)
+{
+  ldfile_open_command_file_1 (name, script_T);
 }
 
 /* Open command file NAME at the default script location.  */
@@ -623,7 +958,7 @@ ldfile_open_command_file (const char *name)
 void
 ldfile_open_default_command_file (const char *name)
 {
-  ldfile_open_command_file_1 (name, TRUE);
+  ldfile_open_command_file_1 (name, script_defaultT);
 }
 
 void
@@ -647,6 +982,18 @@ ldfile_add_arch (const char *in_name)
 
 }
 
+static void
+ldfile_arch_free (search_arch_type **root)
+{
+  search_arch_type *ent;
+  while ((ent = *root) != NULL)
+    {
+      *root = ent->next;
+      free (ent->name);
+      free (ent);
+    }
+}
+
 /* Set the output architecture.  */
 
 void
@@ -663,5 +1010,19 @@ ldfile_set_output_arch (const char *string, enum bfd_architecture defarch)
   else if (defarch != bfd_arch_unknown)
     ldfile_output_architecture = defarch;
   else
-    einfo (_("%F%P: cannot represent machine `%s'\n"), string);
+    fatal (_("%P: cannot represent machine `%s'\n"), string);
+}
+
+/* Tidy up memory.  */
+
+void
+ldfile_free (void)
+{
+  ldfile_remap_input_free ();
+  ldfile_library_path_free (&script_search);
+  search_tail_ptr = &search_head;
+  ldfile_library_path_free (&search_head);
+  search_arch_tail_ptr = &search_arch_head;
+  ldfile_arch_free (&search_arch_head);
+  ldfile_script_free (&processed_scripts);
 }
